#include "Environment.h"
#include "RenderSampler.h"
#include "GlRenderer.h"

namespace GtpVisibilityPreprocessor {


RenderSampler::RenderSampler():Preprocessor()
{
  environment->GetIntValue("RenderSampler.samples", mSamples);
  cout << "number of render samples: " << mSamples << endl;
}

bool
RenderSampler::ComputeVisibility()
{
	long startTime = GetTime();
	
	int histoMaxVal = 0;
	int histoIntervals = 0;
	
    environment->GetIntValue("Preprocessor.histogram.maxValue", histoMaxVal);
	environment->GetIntValue("Preprocessor.histogram.intervals", histoIntervals);

	const int intervals = histoIntervals;

	cout << "starting sampling of render cost ... ";

	vector<RenderCostSample> samples;
  
	if (renderer)
	{cout << "here18 " << mSamples << endl;
		renderer->SampleRenderCost(mSamples, samples);
	}

	cout << "finished in " << TimeDiff(startTime, GetTime()) * 1e-3 << " secs" << endl;


	//-- Evaluate properties
	

	//-- compute histogram from the samples
	int avgPvs = 0;
	int maxPvs = 0;

	vector<int> histogram(intervals);
  
	int i;

	/// initialise histogram
	for (i = 0; i < intervals; ++ i)
	{
		histogram[i] = 0;
	}

	

	for (i = 0; i < mSamples; ++ i) 
	{
		if (samples[i].mVisibleObjects > maxPvs)
		{
			maxPvs = samples[i].mVisibleObjects;
		}

		avgPvs += samples[i].mVisibleObjects;
	}

	// sometimes need need to unifiy maximum pvs to compare with other method's histograms
	// => choose histoMaxPvs accordingly (higher than all pvsss)
	const int maxVal = max(histoMaxVal, maxPvs);
	
	for (i = 0; i < mSamples; ++ i) 
	{
		const int bin = (samples[i].mVisibleObjects * intervals) / maxVal;

		++ histogram[bin];
	}


	//-- output the histogram

	const string filename("fromPointHisto.log");
	std::ofstream outstream;
	outstream.open(filename.c_str());



	Debug << "****************************************" << endl;
	Debug << "From point queries: " << endl;

	for (i = 0; i < intervals; ++ i) 
	{
		outstream << "#Pass\n" << i << endl;
		outstream << "#Pvs\n" << histogram[i] << endl;
		//cout << histogram[i] << endl;
	}

	outstream.close();

	avgPvs /= mSamples;
		
	Debug << "max pvs: " << maxPvs << endl;
	Debug << "avg pvs: " << avgPvs << endl;
  
	return true;
}



}
